@@ -1,7 +1,7 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2008-2014 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2008-2015 -- leonerd@leonerd.org.uk
 
 package IO::Async::Listener;
 
@@ -9,7 +9,7 @@ use strict;
 use warnings;
 use base qw( IO::Async::Handle );
 
-our $VERSION = '0.64';
+our $VERSION = '0.67';
 
 use IO::Async::Handle;
 use IO::Async::OS;
@@ -43,7 +43,7 @@ C<IO::Async::Listener> - listen on network sockets for incoming connections
              return 0;
           },
        );
-       
+
        $loop->add( $stream );
     },
  );
@@ -53,10 +53,7 @@ C<IO::Async::Listener> - listen on network sockets for incoming connections
  $listener->listen(
     service  => "echo",
     socktype => 'stream',
-
-    on_resolve_error => sub { print STDERR "Cannot resolve - $_[0]\n"; },
-    on_listen_error  => sub { print STDERR "Cannot listen\n"; },
- );
+ )->get;
 
  $loop->run;
 
@@ -74,10 +71,7 @@ This object can also be used indirectly via an C<IO::Async::Loop>:
     on_stream => sub {
        ...
     },
-
-    on_resolve_error => sub { print STDERR "Cannot resolve - $_[0]\n"; },
-    on_listen_error  => sub { print STDERR "Cannot listen\n"; },
- );
+ )->get;
 
  $loop->run;
 
@@ -128,7 +122,7 @@ constructor or class.
 
 Optional. Invoked if the C<accept> syscall indicates an error (other than
 C<EAGAIN> or C<EWOULDBLOCK>). If not provided, failures of C<accept> will
-simply be ignored.
+be passed to the main C<on_error> handler.
 
 =cut
 
@@ -294,13 +288,17 @@ sub on_read_ready
       my ( $result ) = @_ or return; # false-alarm
       $on_done->( $self, $result );
    })->on_fail( sub {
-      my ( $message, undef, $socket, $dollarbang ) = @_;
-      $self->maybe_invoke_event( on_accept_error => $socket, $dollarbang );
+      my ( $message, $name, @args ) = @_;
+      if( $name eq "accept" ) {
+         my ( $socket, $dollarbang ) = @args;
+         $self->maybe_invoke_event( on_accept_error => $socket, $dollarbang ) or
+            $self->invoke_error( "accept() failed - $dollarbang", accept => $socket, $dollarbang );
+      }
    });
 
    # Caller is not going to keep hold of the Future, so we have to ensure it
    # stays alive somehow
-   $f->on_ready( sub { undef $f } ); # intentional cycle
+   $self->adopt_future( $f->else( sub { Future->done } ) );
 }
 
 sub _accept
@@ -314,17 +312,17 @@ sub _accept
       $accepted->blocking( 0 );
       if( my $handle = $params{handle} ) {
          $handle->set_handle( $accepted );
-         return Future->new->done( $handle );
+         return Future->done( $handle );
       }
       else {
-         return Future->new->done( $accepted );
+         return Future->done( $accepted );
       }
    }
    elsif( $! == EAGAIN or $! == EWOULDBLOCK ) {
-      return Future->new->done;
+      return Future->done;
    }
    else {
-      return Future->new->fail( "Cannot accept() - $!", accept => $listen_sock, $! );
+      return Future->fail( "Cannot accept() - $!", accept => $listen_sock, $! );
    }
 }
 
@@ -462,7 +460,7 @@ sockets.
              return 0;
           },
        );
-       
+
        $loop->add( $stream );
     },
  );
@@ -510,6 +508,38 @@ earlier example:
     ...
  );
 
+=head2 Using A Kernel-Assigned Port Number
+
+Rather than picking a specific port number, is it possible to ask the kernel
+to assign one arbitrarily that is currently free. This can be done by
+requesting port number 0 (which is actually the default if no port number is
+otherwise specified). To determine which port number the kernel actually
+picked, inspect the C<sockport> accessor on the actual socket filehandle.
+
+Either use the L<Future> returned by the C<listen> method:
+
+ $listener->listen(
+    addr => { family => "inet" },
+ )->on_done( sub {
+    my ( $listener ) = @_;
+    my $socket = $listener->read_handle;
+
+    say "Now listening on port ", $socket->sockport;
+ });
+
+Or pass an C<on_listen> continuation:
+
+ $listener->listen(
+    addr => { family => "inet" },
+
+    on_listen => sub {
+       my ( $listener ) = @_;
+       my $socket = $listener->read_handle;
+
+       say "Now listening on port ", $socket->sockport;
+    },
+ );
+
 =head1 AUTHOR
 
 Paul Evans <leonerd@leonerd.org.uk>
