@@ -1,14 +1,15 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2007-2013 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2007-2015 -- leonerd@leonerd.org.uk
 
 package IO::Async::Loop;
 
 use strict;
 use warnings;
+use 5.010;
 
-our $VERSION = '0.64';
+our $VERSION = '0.67';
 
 # When editing this value don't forget to update the docs below
 use constant NEED_API_VERSION => '0.33';
@@ -78,6 +79,8 @@ $SIG{ALRM} = sub {
    }
 } if WATCHDOG_ENABLE;
 
+$SIG{PIPE} = "IGNORE" if ( $SIG{PIPE} // "" ) eq "DEFAULT";
+
 =head1 NAME
 
 C<IO::Async::Loop> - core loop of the C<IO::Async> framework
@@ -133,6 +136,15 @@ See also the two bundled Loop subclasses:
 Or other subclasses that may appear on CPAN which are not part of the core
 C<IO::Async> distribution.
 
+=head2 Ignoring SIGPIPE
+
+Since version I<0.66> loading this module automatically ignores C<SIGPIPE>, as
+it is highly unlikely that the default-terminate action is the best course of
+action for an C<IO::Async>-based program to take. If at load time the handler
+disposition is still set as C<DEFAULT>, it is set to ignore. If already
+another handler has been placed there by the program code, it will be left
+undisturbed.
+
 =cut
 
 # Internal constructor used by subclasses
@@ -235,12 +247,21 @@ In cases where the module subclass is a hard requirement, such as GTK programs
 using C<Glib>, it would be better to use the module specifically and invoke
 its constructor directly.
 
+=item * IO::Async::OS->LOOP_PREFER_CLASSES
+
+The L<IO::Async::OS> hints module for the given OS is then consulted to see if
+it suggests any other module classes specific to the given operating system.
+
 =item * $^O
 
 The module called C<IO::Async::Loop::$^O> is tried next. This allows specific
 OSes, such as the ever-tricky C<MSWin32>, to provide an implementation that
 might be more efficient than the generic ones, or even work at all.
 
+This option is now discouraged in favour of the C<IO::Async::OS> hint instead.
+At some future point it may be removed entirely, given as currently only
+C<linux> uses it.
+
 =item * Poll and Select
 
 Finally, if no other choice has been made by now, the built-in C<Poll> module
@@ -309,7 +330,14 @@ sub really_new
       warn "Unable to use $class - $topline\n";
    }
 
-   $self = __try_new( "IO::Async::Loop::$^O" ) and return $self unless $LOOP_NO_OS;
+   unless( $LOOP_NO_OS ) {
+      foreach my $class ( IO::Async::OS->LOOP_PREFER_CLASSES, "IO::Async::Loop::$^O" ) {
+         $class =~ m/::/ or $class = "IO::Async::Loop::$class";
+         $self = __try_new( $class ) and return $self;
+
+         # Don't complain about these ones
+      }
+   }
 
    return IO::Async::Loop->new_builtin;
 }
@@ -530,6 +558,25 @@ sub loop_stop
    $self->stop;
 }
 
+=head2 $loop->post_fork
+
+The base implementation of this method does nothing. It is provided in case
+some Loop subclasses should take special measures after a C<fork()> system
+call if the main body of the program should survive in both running processes.
+
+This may be required, for example, in a long-running server daemon that forks
+multiple copies on startup after opening initial listening sockets. A loop
+implementation that uses some in-kernel resource that becomes shared after
+forking (for example, a Linux C<epoll> or a BSD C<kqueue> filehandle) would
+need recreating in the new child process before the program can continue.
+
+=cut
+
+sub post_fork
+{
+   # empty
+}
+
 ###########
 # Futures #
 ###########
@@ -1062,6 +1109,8 @@ sub set_resolver
       for qw( resolve getaddrinfo getnameinfo );
 
    $self->{resolver} = $resolver;
+
+   $self->add( $resolver );
 }
 
 =head2 @result = $loop->resolve( %params )->get
@@ -1357,7 +1406,7 @@ sub connect
 
    $future = $future->then( sub {
       $handle->set_handle( shift );
-      return Future->new->done( $handle )
+      return Future->done( $handle )
    }) if $handle;
 
    $future->on_done( $on_done ) if $on_done;
@@ -2011,6 +2060,10 @@ existing one of that type. It is not required that both are provided.
 Applications should use a C<IO::Async::Handle> or C<IO::Async::Stream> instead
 of using this method.
 
+If the filehandle does not yet have the C<O_NONBLOCK> flag set, it will be
+enabled by this method. This will ensure that any subsequent C<sysread>,
+C<syswrite>, or similar will not block on the filehandle.
+
 =cut
 
 # This class specifically does NOT implement this method, so that subclasses
@@ -2021,6 +2074,10 @@ sub __watch_io
    my %params = @_;
 
    my $handle = delete $params{handle} or croak "Expected 'handle'";
+   defined eval { $handle->fileno } or croak "Expected that 'handle' has defined ->fileno";
+
+   # Silent "upgrade" to O_NONBLOCK
+   $handle->blocking and $handle->blocking(0);
 
    my $watch = ( $self->{iowatches}->{$handle->fileno} ||= [] );
 
