@@ -1,14 +1,14 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2011 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2011-2015 -- leonerd@leonerd.org.uk
 
 package IO::Async::Socket;
 
 use strict;
 use warnings;
 
-our $VERSION = '0.61';
+our $VERSION = '0.67';
 
 use base qw( IO::Async::Handle );
 
@@ -28,36 +28,27 @@ filehandle
  use IO::Async::Loop;
  my $loop = IO::Async::Loop->new;
 
- $loop->connect(
+ my $socket = IO::Async::Socket->new(
+    on_recv => sub {
+       my ( $self, $dgram, $addr ) = @_;
+
+       print "Received reply: $dgram\n",
+       $loop->stop;
+    },
+    on_recv_error => sub {
+       my ( $self, $errno ) = @_;
+       die "Cannot recv - $errno\n";
+    },
+ );
+ $loop->add( $socket );
+
+ $socket->connect(
     host     => "some.host.here",
     service  => "echo",
     socktype => 'dgram',
+ )->get;
 
-    on_connected => sub {
-       my ( $sock ) = @_;
-
-       my $socket = IO::Async::Socket->new(
-          handle => $sock,
-          on_recv => sub {
-             my ( $self, $dgram, $addr ) = @_;
-
-             print "Received reply: $dgram\n",
-             $loop->stop;
-          },
-          on_recv_error => sub {
-             my ( $self, $errno ) = @_;
-             die "Cannot recv - $errno\n";
-          },
-       );
-
-       $loop->add( $socket );
-
-       $socket->send( "A TEST DATAGRAM" );
-    },
-
-    on_resolve_error => sub { die "Cannot resolve - $_[0]\n"; },
-    on_connect_error => sub { die "Cannot connect\n"; },
- );
+ $socket->send( "A TEST DATAGRAM" );
 
  $loop->run;
 
@@ -68,9 +59,9 @@ provides a queue of outgoing data. It invokes the C<on_recv> handler when new
 data is received from the filehandle. Data may be sent to the filehandle by
 calling the C<send> method.
 
-It is primarily intended for C<SOCK_DGRAM> or C<SOCK_RAW> sockets; for
-C<SOCK_STREAM> sockets an instance of L<IO::Async::Stream> is probably more
-appropriate.
+It is primarily intended for C<SOCK_DGRAM> or C<SOCK_RAW> sockets (such as UDP
+or packet-capture); for C<SOCK_STREAM> sockets (such as TCP) an instance of
+L<IO::Async::Stream> is more appropriate.
 
 =head1 EVENTS
 
@@ -121,39 +112,37 @@ sub _init
 
 The following named parameters may be passed to C<new> or C<configure>:
 
-=over 8
-
-=item read_handle => IO
+=head2 read_handle => IO
 
 The IO handle to receive from. Must implement C<fileno> and C<recv> methods.
 
-=item write_handle => IO
+=head2 write_handle => IO
 
 The IO handle to send to. Must implement C<fileno> and C<send> methods.
 
-=item handle => IO
+=head2 handle => IO
 
 Shortcut to specifying the same IO handle for both of the above.
 
-=item on_recv => CODE
+=head2 on_recv => CODE
 
-=item on_recv_error => CODE
+=head2 on_recv_error => CODE
 
-=item on_outgoing_empty => CODE
+=head2 on_outgoing_empty => CODE
 
-=item on_send_error => CODE
+=head2 on_send_error => CODE
 
-=item autoflush => BOOL
+=head2 autoflush => BOOL
 
 Optional. If true, the C<send> method will atempt to send data to the
 operating system immediately, without waiting for the loop to indicate the
 filehandle is write-ready.
 
-=item recv_len => INT
+=head2 recv_len => INT
 
 Optional. Sets the buffer size for C<recv> calls. Defaults to 64 KiB.
 
-=item recv_all => BOOL
+=head2 recv_all => BOOL
 
 Optional. If true, repeatedly call C<recv> when the receiving handle first
 becomes read-ready. By default this is turned off, meaning at most one
@@ -166,14 +155,12 @@ filehandles of processing time. Turning this option on may improve bulk data
 transfer rate, at the risk of delaying or stalling processing on other
 filehandles.
 
-=item send_all => INT
+=head2 send_all => INT
 
 Optional. Analogous to the C<recv_all> option, but for sending. When
 C<autoflush> is enabled, this option only affects deferred sending if the
 initial attempt failed.
 
-=back
-
 The condition requiring an C<on_recv> handler is checked at the time the
 object is added to a Loop; it is allowed to create a C<IO::Async::Socket>
 object with a read handle but without a C<on_recv> handler, provided that
@@ -328,6 +315,30 @@ sub on_write_ready
    }
 }
 
+=head1 EXAMPLES
+
+=head2 Send-first on a UDP Socket
+
+C<UDP> is carried by the C<SOCK_DGRAM> socket type, for which the string
+C<'dgram'> is a convenient shortcut:
+
+ $socket->connect(
+    host     => $hostname,
+    service  => $service,
+    socktype => 'dgram',
+    ...
+ )
+
+=head2 Receive-first on a UDP Socket
+
+A typical server pattern with C<UDP> involves binding a well-known port
+number instead of connecting to one, and waiting on incoming packets.
+
+ $socket->bind(
+    service  => 12345,
+    socktype => 'dgram',
+ )->get;
+
 =head1 SEE ALSO
 
 =over 4
